#tag Module
Protected Module common
	#tag Method, Flags = &h0
		Sub Chirp(intChirps As Integer = 1)
		  
		  Dim i As Integer
		  
		  
		  ' Chirp the piezo.
		  '
		  For i = 1 To intChirps
		    GPIO.DigitalWrite(gstrConfig_PinBeeper, GPIO.ON)
		    GPIO.Delay(25)
		    GPIO.DigitalWrite(gstrConfig_PinBeeper, GPIO.OFF)
		    GPIO.Delay(25)
		  Next i
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub ConfigLoad(strINI_File As String)
		  
		  Dim objFolderItem As FolderItem
		  Dim objINI As INI
		  
		  
		  ' Get folder item for INI file
		  '
		  objFolderItem = New FolderItem(strINI_File)
		  
		  ' Create INI object.
		  '
		  objINI = New INI(objFolderItem)
		  
		  ' Load configuration.
		  '
		  objINI.Load
		  
		  ' Copy INI to global vars.
		  '
		  gstrConfig_MusicPath = objINI.Get("Media", "MusicPath",  "")
		  gstrConfig_MoviePath = objINI.Get("Media", "MoviePath",  "")
		  gstrConfig_DBaseFile = objINI.Get("DBase", "DBaseFile",  "")
		  gstrConfig_PinBeeper = objINI.Get("GPIO",  "pin_beeper", 26)
		  
		  ' Destroy INI object.
		  '
		  objINI = Nil
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub CrashReport(objError As RuntimeException)
		  
		  Dim strAppPath As String
		  Dim strAppName As String
		  Dim objFolderItem As FolderItem
		  Dim strErrorType As String
		  Dim strDumpTxt As String
		  Dim objDate As Date
		  Dim objTextOutputStream As TextOutputStream
		  Dim strStages() As String = Array("Development", "Alpha", "Beta", "Final") 
		  
		  
		  ' Get current date.
		  '
		  objDate = Date.Now
		  
		  ' Get path to our executable.
		  '
		  strAppPath = GetFolderItem("").NativePath
		  
		  ' Get our executable file name.
		  '
		  strAppName = App.ExecutableFile.Name
		  
		  objFolderItem = GetFolderItem(strAppPath + strAppName + ".err", FolderItem.PathTypeNative)
		  
		  strErrorType = Introspection.GetType(objError).Name
		  
		  strDumpTxt = ""
		  strDumpTxt = strDumpTxt + Repeat("=", 120)                                                                     + EndOfLine
		  strDumpTxt = strDumpTxt + "       Error Date: " + objDate.Totext                                               + EndOfLine
		  strDumpTxt = strDumpTxt + "       Error Type: " + strErrorType                                                 + EndOfLine
		  strDumpTxt = strDumpTxt + "     Error Number: " + objError.ErrorNumber.ToText                                  + EndOfLine
		  strDumpTxt = strDumpTxt + "    Error Message: " + LinuxSysErrorLookup(objError.ErrorNumber)                    + EndOfLine
		  strDumpTxt = strDumpTxt + "      App Version: " + app.MajorVersion.ToText                                      + "."
		  strDumpTxt = strDumpTxt +                         app.MinorVersion.ToText                                      + "." 
		  strDumpTxt = strDumpTxt +                         app.BugVersion.ToText                                        + "." 
		  strDumpTxt = strDumpTxt +                         app.NonReleaseVersion.ToText                                 + EndOfLine
		  strDumpTxt = strDumpTxt + "   App Stage Code: " + app.StageCode.ToText + " (" + strStages(app.StageCode) + ")" + EndOfLine
		  strDumpTxt = strDumpTxt + "App Short Version: " + app.ShortVersion                                             + EndOfLine
		  strDumpTxt = strDumpTxt + " App Long Version: " + app.LongVersion                                              + EndOfLine
		  strDumpTxt = strDumpTxt + "       Build Date: " + app.BuildDate.SQLDateTime                                    + EndOfLine
		  strDumpTxt = strDumpTxt                                                                                        + EndOfLine
		  strDumpTxt = strDumpTxt + "       Call Stack: "                                                                + EndOfLine
		  
		  For Each Element As String In objError.stack
		    strDumpTxt = strDumpTxt + Repeat(" ", 19) + Element + EndOfLine
		  Next
		  
		  objTextOutputStream = TextOutputStream.Append(objFolderItem)
		  objTextOutputStream.Write ConvertEncoding(strDumpTxt, Encodings.ASCII)
		  objTextOutputStream.Close
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub CreateMovieTable(objSQLiteDatabase As SQLiteDatabase)
		  
		  Dim strSQL As String
		  
		  
		  ' Create movie files table.
		  '
		  strSQL = "CREATE TABLE `MovieFiles` " _
		  + "(" _
		  + "  `ID`           INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, " _
		  + "  `File`         TEXT, " _
		  + "  `LastModified` TEXT, " _
		  + "  `MovieArt`     BLOB, " _
		  + "  `MovieName`    TEXT COLLATE NOCASE, " _
		  + "  `MovieYear`    INTEGER, " _
		  + "  `MovieGenre`   TEXT COLLATE NOCASE, " _
		  + "  `MovieTagline` TEXT COLLATE NOCASE" _
		  + ");"
		  
		  objSQLiteDatabase.SQLExecute(strSQL)
		  
		  If objSQLiteDatabase.Error Then
		    MsgBox("Create MovieFiles Table error: " + objSQLiteDatabase.ErrorMessage)
		    Quit
		  End If
		  
		  ' Create indexes.
		  '
		  strSQL = "CREATE INDEX `idxMovieName`    ON `MovieFiles` (`MovieName`    COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxMovieYear`    ON `MovieFiles` (`MovieYear`    COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxMovieGenre`   ON `MovieFiles` (`MovieGenre`   COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxMovieTagline` ON `MovieFiles` (`MovieTagline` COLLATE NOCASE ASC);"
		  
		  objSQLiteDatabase.SQLExecute(strSQL)
		  
		  If objSQLiteDatabase.Error Then
		    MsgBox("Create MovieFiles Indexes error: " + objSQLiteDatabase.ErrorMessage)
		    Quit
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub CreateMusicTable(objSQLiteDatabase As SQLiteDatabase)
		  
		  Dim strSQL As String
		  
		  
		  ' Create music files table.
		  '
		  strSQL = "CREATE TABLE `MusicFiles` " _
		  + "(" _
		  + "  `ID`           INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE, " _
		  + "  `File`         TEXT, "                _
		  + "  `LastModified` TEXT, "                _
		  + "  `AlbumArt`     BLOB, "                _
		  + "  `AlbumName`    TEXT COLLATE NOCASE, " _
		  + "  `AlbumArtist`  TEXT COLLATE NOCASE, " _
		  + "  `AlbumYear`    INTEGER, "             _
		  + "  `AlbumGenre`   TEXT COLLATE NOCASE, " _
		  + "  `DiscNumber`   INTEGER, "             _
		  + "  `TrackNumber`  INTEGER, "             _
		  + "  `TrackName`    TEXT COLLATE NOCASE, " _
		  + "  `TrackArtist`  TEXT COLLATE NOCASE"   _
		  + ");"
		  
		  objSQLiteDatabase.SQLExecute(strSQL)
		  
		  If objSQLiteDatabase.Error Then
		    MsgBox("Create MusicFiles Table error: " + objSQLiteDatabase.ErrorMessage)
		    Quit
		  End If
		  
		  ' Create indexes.
		  '
		  strSQL = "CREATE INDEX `idxAlbumName`   ON `MusicFiles` (`AlbumName`   COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxAlbumArtist` ON `MusicFiles` (`AlbumArtist` COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxAlbumYear`   ON `MusicFiles` (`AlbumYear`   COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxAlbumGenre`  ON `MusicFiles` (`AlbumGenre`  COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxDiscNumber`  ON `MusicFiles` (`DiscNumber`  COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxTrackNumber` ON `MusicFiles` (`TrackNumber` COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxTrackName`   ON `MusicFiles` (`TrackName`   COLLATE NOCASE ASC);" _
		  +      "CREATE INDEX `idxTrackArtist` ON `MusicFiles` (`TrackArtist` COLLATE NOCASE ASC);"
		  
		  objSQLiteDatabase.SQLExecute(strSQL)
		  
		  If objSQLiteDatabase.Error Then
		    MsgBox("Create MusicFiles Indexes error: " + objSQLiteDatabase.ErrorMessage)
		    Quit
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub DropMovieTable(objSQLiteDatabase As SQLiteDatabase)
		  
		  Dim strSQL As String
		  
		  
		  ' Delete existing movie files table.
		  '
		  strSQL = "DROP TABLE IF EXISTS `MovieFiles`;"
		  
		  objSQLiteDatabase.SQLExecute(strSQL)
		  
		  If objSQLiteDatabase.Error Then
		    MsgBox("Drop MovieFiles Table error: " + objSQLiteDatabase.ErrorMessage)
		    Quit
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub DropMusicTable(objSQLiteDatabase As SQLiteDatabase)
		  
		  Dim strSQL As String
		  
		  
		  ' Delete existing music files table.
		  '
		  strSQL = "DROP TABLE IF EXISTS `MusicFiles`;"
		  
		  objSQLiteDatabase.SQLExecute(strSQL)
		  
		  If objSQLiteDatabase.Error Then
		    MsgBox("Drop MusicFiles Table error: " + objSQLiteDatabase.ErrorMessage)
		    Quit
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function LinuxSysErrorLookup(intErrNum As Integer) As String
		  
		  Dim strSysErrors(133) As String
		  
		  strSysErrors(001) = "Operation not permitted"
		  strSysErrors(002) = "No such file or directory"
		  strSysErrors(003) = "No such process"
		  strSysErrors(004) = "Interrupted system call"
		  strSysErrors(005) = "Input/output error"
		  strSysErrors(006) = "No such device or address"
		  strSysErrors(007) = "Argument list too long"
		  strSysErrors(008) = "Exec format error"
		  strSysErrors(009) = "Bad file descriptor"
		  strSysErrors(010) = "No child processes"
		  strSysErrors(011) = "Resource temporarily unavailable"
		  strSysErrors(012) = "Cannot allocate memory"
		  strSysErrors(013) = "Permission denied"
		  strSysErrors(014) = "Bad address"
		  strSysErrors(015) = "Block device required"
		  strSysErrors(016) = "Device or resource busy"
		  strSysErrors(017) = "File exists"
		  strSysErrors(018) = "Invalid cross-device link"
		  strSysErrors(019) = "No such device"
		  strSysErrors(020) = "Not a directory"
		  strSysErrors(021) = "Is a directory"
		  strSysErrors(022) = "Invalid argument"
		  strSysErrors(023) = "File table overflow"
		  strSysErrors(024) = "Too many open files"
		  strSysErrors(025) = "Inappropriate ioctl for device"
		  strSysErrors(026) = "Text file busy"
		  strSysErrors(027) = "File too large"
		  strSysErrors(028) = "No space left on device"
		  strSysErrors(029) = "Illegal seek"
		  strSysErrors(030) = "Read-only file system"
		  strSysErrors(031) = "Too many links"
		  strSysErrors(032) = "Broken pipe"
		  strSysErrors(033) = "Numerical argument out of domain"
		  strSysErrors(034) = "Numerical result out of range"
		  strSysErrors(035) = "Resource deadlock avoided"
		  strSysErrors(036) = "File name too long"
		  strSysErrors(037) = "No record locks available"
		  strSysErrors(038) = "Function not implemented"
		  strSysErrors(039) = "Directory not empty"
		  strSysErrors(040) = "Too many levels of symbolic links"
		  strSysErrors(041) = "?"
		  strSysErrors(042) = "No message of desired type"
		  strSysErrors(043) = "Identifier removed"
		  strSysErrors(044) = "Channel number out of range"
		  strSysErrors(045) = "Level 2 not synchronized"
		  strSysErrors(046) = "Level 3 halted"
		  strSysErrors(047) = "Level 3 reset"
		  strSysErrors(048) = "Link number out of range"
		  strSysErrors(049) = "Protocol driver not attached"
		  strSysErrors(050) = "No CSI structure available"
		  strSysErrors(051) = "Level 2 halted"
		  strSysErrors(052) = "Invalid exchange"
		  strSysErrors(053) = "Invalid request descriptor"
		  strSysErrors(054) = "Exchange full"
		  strSysErrors(055) = "No anode"
		  strSysErrors(056) = "Invalid request code"
		  strSysErrors(057) = "Invalid slot"
		  strSysErrors(058) = "?"
		  strSysErrors(059) = "Bad font file format"
		  strSysErrors(060) = "Device not a stream"
		  strSysErrors(061) = "No data available"
		  strSysErrors(062) = "Timer expired"
		  strSysErrors(063) = "Out of streams resources"
		  strSysErrors(064) = "Machine is not on the network"
		  strSysErrors(065) = "Package not installed"
		  strSysErrors(066) = "Object is remote"
		  strSysErrors(067) = "Link has been severed"
		  strSysErrors(068) = "Advertise error"
		  strSysErrors(069) = "Srmount error"
		  strSysErrors(070) = "Communication error on send"
		  strSysErrors(071) = "Protocol error"
		  strSysErrors(072) = "Multihop attempted"
		  strSysErrors(073) = "RFS specific error"
		  strSysErrors(074) = "Bad message"
		  strSysErrors(075) = "Value too large for defined data type"
		  strSysErrors(076) = "Name not unique on network"
		  strSysErrors(077) = "File descriptor in bad state"
		  strSysErrors(078) = "Remote address changed"
		  strSysErrors(079) = "Can not access a needed shared library"
		  strSysErrors(080) = "Accessing a corrupted shared library"
		  strSysErrors(081) = ".lib section in a.out corrupted"
		  strSysErrors(082) = "Attempting to link in too many shared libraries"
		  strSysErrors(083) = "Cannot exec a shared library directly"
		  strSysErrors(084) = "Invalid or incomplete multibyte or wide character"
		  strSysErrors(085) = "Interrupted system call should be restarted"
		  strSysErrors(086) = "Streams pipe error"
		  strSysErrors(087) = "Too many users"
		  strSysErrors(088) = "Socket operation on non-socket"
		  strSysErrors(089) = "Destination address required"
		  strSysErrors(090) = "Message too long"
		  strSysErrors(091) = "Protocol wrong type for socket"
		  strSysErrors(092) = "Protocol not available"
		  strSysErrors(093) = "Protocol not supported"
		  strSysErrors(094) = "Socket type not supported"
		  strSysErrors(095) = "Operation not supported"
		  strSysErrors(096) = "Protocol family not supported"
		  strSysErrors(097) = "Address family not supported by protocol"
		  strSysErrors(098) = "Address already in use"
		  strSysErrors(099) = "Cannot assign requested address"
		  strSysErrors(100) = "Network is down"
		  strSysErrors(101) = "ENETUNREACH Network is unreachable"
		  strSysErrors(102) = "Network dropped connection on reset"
		  strSysErrors(103) = "Software caused connection abort"
		  strSysErrors(104) = "Connection reset by peer"
		  strSysErrors(105) = "No buffer space available"
		  strSysErrors(106) = "Transport endpoint is already connected"
		  strSysErrors(107) = "Transport endpoint is not connected"
		  strSysErrors(108) = "Cannot send after transport endpoint shutdown"
		  strSysErrors(109) = "Too many references: cannot splice"
		  strSysErrors(110) = "Connection timed out"
		  strSysErrors(111) = "Connection refused"
		  strSysErrors(112) = "Host is down"
		  strSysErrors(113) = "No route to host"
		  strSysErrors(114) = "Operation already in progress"
		  strSysErrors(115) = "Operation now in progress"
		  strSysErrors(116) = "Stale file handle"
		  strSysErrors(117) = "Structure needs cleaning"
		  strSysErrors(118) = "Not a XENIX named type file"
		  strSysErrors(119) = "No XENIX semaphores available"
		  strSysErrors(120) = "Is a named type file"
		  strSysErrors(121) = "Remote I/O error"
		  strSysErrors(122) = "Disk quota exceeded"
		  strSysErrors(123) = "No medium found"
		  strSysErrors(124) = "Wrong medium type"
		  strSysErrors(125) = "Operation canceled"
		  strSysErrors(126) = "Required key not available"
		  strSysErrors(127) = "Key has expired"
		  strSysErrors(128) = "Key has been revoked"
		  strSysErrors(129) = "Key was rejected by service"
		  strSysErrors(130) = "Owner died"
		  strSysErrors(131) = "State not recoverable"
		  strSysErrors(132) = "Operation not possible due to RF-kill"
		  strSysErrors(133) = "Memory page has hardware error"
		  
		  If (intErrNum >= 0) And (intErrNum <= strSysErrors.Ubound) Then
		    Return  strSysErrors(intErrNum)
		  Else
		    Return "Error lookup failed."
		  End If
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function OpenDatabase(strDBaseFile As String) As SQLiteDatabase
		  
		  Dim objSQLiteDatabase As SQLiteDatabase
		  Dim objFolderItem As FolderItem
		  Dim strSQL As String
		  
		  
		  ' Get folder item for the database file.
		  '
		  objFolderItem = GetFolderItem(strDBaseFile)
		  
		  ' Create database or connect if already exists.
		  '
		  objSQLiteDatabase = New SQLiteDatabase
		  objSQLiteDatabase.DatabaseFile = objFolderItem
		  
		  If objSQLiteDatabase.CreateDatabaseFile = False Then
		    MsgBox("Create Database File error: " + objSQLiteDatabase.ErrorMessage)
		    Quit
		  End If
		  
		  Return objSQLiteDatabase
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Repeat(s As String, n As Integer) As String
		  
		  Dim strTemp As String
		  
		  If n <= 0 Then Return ""
		  If n  = 1 Then Return s
		  
		  strTemp = ""
		  For Index As Integer = 1 To n
		    strTemp = strTemp + s
		  Next Index
		  
		  Return strTemp
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function SafeParseJSON(txtJSON As Text) As Auto
		  #Pragma BreakOnExceptions Off
		  
		  Try
		    
		    Return ParseJSON(txtJSON)
		    
		  Catch objError As InvalidJSONException
		    
		    Return New Dictionary
		    
		  Catch objError As RunTimeException
		    
		    Raise objError
		    
		  End Try
		  
		  #Pragma BreakOnExceptions On
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function SglQuote(s As String) As String
		  
		  ' Returns the supplied string wrapped in single-quotes.
		  '
		  Return Chr(39) + s + Chr(39)
		  
		End Function
	#tag EndMethod


	#tag Property, Flags = &h0
		genuCatalog As Catalog
	#tag EndProperty

	#tag Property, Flags = &h0
		gobjMainWindow As main_window = Nil
	#tag EndProperty

	#tag Property, Flags = &h0
		gpicSource As Picture = Nil
	#tag EndProperty

	#tag Property, Flags = &h0
		gstrAppName As String
	#tag EndProperty

	#tag Property, Flags = &h0
		gstrAppPath As String
	#tag EndProperty

	#tag Property, Flags = &h0
		gstrConfig_DBaseFile As String
	#tag EndProperty

	#tag Property, Flags = &h0
		gstrConfig_MoviePath As String
	#tag EndProperty

	#tag Property, Flags = &h0
		gstrConfig_MusicPath As String
	#tag EndProperty

	#tag Property, Flags = &h0
		gstrConfig_PinBeeper As Integer
	#tag EndProperty


	#tag Enum, Name = Catalog, Type = Integer, Flags = &h0
		MusicFiles
		MovieFiles
	#tag EndEnum

	#tag Using, Name = Xojo.Core
	#tag EndUsing

	#tag Using, Name = Xojo.Data
	#tag EndUsing


	#tag ViewBehavior
		#tag ViewProperty
			Name="gstrAppName"
			Group="Behavior"
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="gstrAppPath"
			Group="Behavior"
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="gstrConfig_DBaseFile"
			Group="Behavior"
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="gstrConfig_MoviePath"
			Group="Behavior"
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="gstrConfig_MusicPath"
			Group="Behavior"
			Type="String"
			EditorType="MultiLineEditor"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="gpicSource"
			Group="Behavior"
			InitialValue="Nil"
			Type="Picture"
		#tag EndViewProperty
		#tag ViewProperty
			Name="genuCatalog"
			Group="Behavior"
			Type="Catalog"
			EditorType="Enum"
			#tag EnumValues
				"0 - MusicFiles"
				"1 - MovieFiles"
			#tag EndEnumValues
		#tag EndViewProperty
		#tag ViewProperty
			Name="genuOperation"
			Group="Behavior"
			Type="Operation"
			EditorType="Enum"
			#tag EnumValues
				"0 - Update"
				"1 - Rebuild"
			#tag EndEnumValues
		#tag EndViewProperty
		#tag ViewProperty
			Name="gstrConfig_PinBeeper"
			Group="Behavior"
			Type="Integer"
		#tag EndViewProperty
	#tag EndViewBehavior
End Module
#tag EndModule
