#tag Class
Protected Class music_player_mgr
	#tag DelegateDeclaration, Flags = &h21
		Private Delegate Sub Callback()
	#tag EndDelegateDeclaration

	#tag Method, Flags = &h0
		Sub Constructor(MethodToCall As Callback)
		  
		  ' Assign the method passed to us by the instantiator to our delegate
		  ' object.
		  '
		  pobjCallback = MethodToCall
		  
		  ' Create a timer to watch the OMXPlayer to detect if it has finished
		  ' playing.
		  '
		  ptmrOMXWatchdog = New Timer
		  AddHandler ptmrOMXWatchdog.Action, WeakAddressOf Self.OMXWatchdog_Action
		  ptmrOMXWatchdog.Mode = Timer.ModeMultiple
		  ptmrOMXWatchdog.Period = 1000
		  ptmrOMXWatchdog.Enabled = False
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Destructor()
		  
		  ' Shutdown and destroy timer object.
		  '
		  ptmrOMXWatchdog.Enabled = False
		  RemoveHandler ptmrOMXWatchdog.Action, WeakAddressOf Self.OMXWatchdog_Action
		  ptmrOMXWatchdog = Nil
		  
		  ' Destroy callback reference.
		  '
		  pobjCallback = Nil
		  
		  ' Shutdown OMXPlayer if it is running.
		  '
		  If (penuState = States.Playing) Or (penuState = States.Paused) Then
		    penuState = States.Stopping
		    Call OMXCommand("stop")
		    OMXWaitForReady(False)
		    penuState = States.Stopped
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Duration() As UInt64
		  
		  Return pintDuration
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function LastFilePlayed() As String
		  
		  Return pstrLastFilePlayed
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function OMXCommand(strCommand As String) As String
		  
		  Dim objShell As Shell
		  Dim strRetVal As String
		  
		  
		  ' Creates a temporary shell so we can invoke the omx_player_dbus_control.sh script and send DBUS commands
		  ' to the OMXPlayer.
		  
		  objShell = New Shell
		  
		  objShell.Execute(gstrAppPath + "omx_player_dbus_control.sh" + " " + DblQuote("comvette.music.player") + " " + strCommand)
		  strRetVal = objShell.Result
		  
		  objShell = Nil
		  
		  Return strRetVal
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub OMXWaitForReady(booReady As Boolean)
		  
		  Dim strRetVal As String
		  Dim intStartTicks As Integer
		  Dim intWaitTicks As Integer
		  
		  
		  ' Wait for 20 seconds.
		  '
		  intWaitTicks = 20 * 60
		  
		  ' Get tick count at start.
		  '
		  intStartTicks = Ticks
		  
		  ' Begin forever loop.
		  '
		  Do
		    
		    ' Poll the OMXPlayer for the media position. If it returns a numeric string we
		    ' know the player is fully instantiated and can be controlled.
		    '
		    strRetVal = OMXCommand("position")
		    If IsNumeric(RTrim(strRetVal)) = booReady Then
		      Exit Do
		    End If
		    
		    ' Have we tried for more than 20 seconds to get a response?
		    '
		    If Ticks > (intStartTicks + intWaitTicks) Then
		      AckBox "Timeout Error", "OMXPlayer failed to respond within 20 seconds.", True
		      Quit
		    End If
		    
		  Loop
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub OMXWatchdog_Action(tmrInstance As Timer)
		  
		  Dim strRetVal As String
		  Dim intPosition As UInt64
		  
		  
		  ' Poll the OMXPlayer for the media position (when it stops returning a numeric string 
		  ' we know the player has finished playing) and then invoke our callback to send a
		  ' completed notification to the parent of this class.
		  
		  strRetVal = RTrim(OMXCommand("position"))
		  intPosition = CType(Val(strRetVal), UInt64)
		  
		  System.DebugLog(Str(intPosition \ 1000000) + " of " + Str(pintDuration \ 1000000) + " Seconds")
		  
		  If IsNumeric(RTrim(strRetVal)) = False Then
		    ptmrOMXWatchdog.Enabled = False
		    penuState = States.Stopped
		    pobjCallback.Invoke
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Pause()
		  
		  ' Pause playing.
		  '
		  If penuState = States.Playing Then
		    penuState = States.Pausing
		    ptmrOMXWatchdog.Enabled = False
		    Call OMXCommand("pause")
		    penuState = States.Paused
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Play(strFile As String = "")
		  
		  ' Play method logic:
		  '
		  ' If the Music player state is "stopped", then play the provided file. If no file was provided, 
		  ' then exit the Play method.
		  '
		  ' If the Music player state is "paused", then resume playing the current file.
		  '
		  ' If the Music player state is "playing", then stop playing the current file. If a file was
		  ' provided, then play the provided file, otherwise restart the current file.
		  '
		  Select Case penuState
		    
		  Case States.Stopped
		    
		    ' Start playing new media file if provided.
		    '
		    If strFile <> "" Then
		      
		      penuState = States.Starting
		      WriteMusicLauncherData(strFile)
		      pintDuration = Val(OMXCommand("duration"))
		      ptmrOMXWatchdog.Enabled = True
		      penuState = States.Playing
		      
		      pstrLastFilePlayed = strFile
		      
		    End If
		    
		  Case States.Paused
		    
		    ' Resume playing current media file.
		    '
		    penuState = States.Resuming
		    Call OMXCommand("play")
		    ptmrOMXWatchdog.Enabled = True
		    penuState = States.Playing
		    
		  Case States.Playing
		    
		    ' Stop playing current media file.
		    '
		    penuState = States.Stopping
		    ptmrOMXWatchdog.Enabled = False
		    Call OMXCommand("stop")
		    OMXWaitForReady(False)
		    penuState = States.Stopped
		    
		    ' Start playing new media file if provided. Otherwise restart
		    ' current media file.
		    '
		    If strFile <> "" Then
		      
		      penuState = States.Starting
		      WriteMusicLauncherData(strFile)
		      pintDuration = Val(OMXCommand("duration"))
		      ptmrOMXWatchdog.Enabled = True
		      penuState = States.Playing
		      
		      pstrLastFilePlayed = strFile
		      
		    Else
		      
		      penuState = States.Starting
		      WriteMusicLauncherData(pstrLastFilePlayed)
		      ptmrOMXWatchdog.Enabled = True
		      penuState = States.Playing
		      
		    End If
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function State() As music_player_mgr.States
		  
		  Return penuState
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Stop()
		  
		  ' Stop playing.
		  '
		  If (penuState = States.Playing) Or (penuState = States.Paused) Then
		    penuState = States.Stopping
		    ptmrOMXWatchdog.Enabled = False
		    Call OMXCommand("stop")
		    penuState = States.Stopped
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub WriteMusicLauncherData(strFile As String)
		  
		  Dim objFolderItem As FolderItem
		  Dim objTextOutputStream As TextOutputStream
		  
		  
		  ' Creates a launch_music.dat file and writes the name of the music file to be played
		  ' into it. 
		  '
		  ' The launch_music.sh file is always running as a background process waiting for a 
		  ' launch_music.dat file to appear. When it finds a launch_music.dat file it will
		  ' launch the OMXPlayer with the music file named in the launch_music.dat file.
		  '
		  ' Once the launch_music.sh process launches the OMXPlayer, it pauses until the 
		  ' OMXPlayer is finished playing the music file at which time it will return to
		  ' waiting for a new launch_music.dat file to appear.
		  
		  
		  ' Create a launch_music.dat file.
		  '
		  objFolderItem = GetFolderItem(gstrAppPath + "launch_music.dat", FolderItem.PathTypeNative)
		  objTextOutputStream = TextOutputStream.Create(objFolderItem)
		  objTextOutputStream.WriteLine(strFile)
		  objTextOutputStream.Close
		  
		  OMXWaitForReady(True)
		  
		  ' Once the OMXPlayer is running, delete the launch_music.dat file. It has served it's
		  ' purpose and is no longer needed.
		  '
		  objFolderItem = GetFolderItem(gstrAppPath + "launch_music.dat", FolderItem.PathTypeNative)
		  objFolderItem.Delete
		  
		End Sub
	#tag EndMethod


	#tag Property, Flags = &h21
		Private penuState As States = States.Stopped
	#tag EndProperty

	#tag Property, Flags = &h21
		Private pintDuration As UInt64 = 0
	#tag EndProperty

	#tag Property, Flags = &h21
		Private pobjCallback As Callback = Nil
	#tag EndProperty

	#tag Property, Flags = &h21
		Private pstrLastFilePlayed As String
	#tag EndProperty

	#tag Property, Flags = &h21
		Private ptmrOMXWatchdog As Timer = Nil
	#tag EndProperty


	#tag Enum, Name = States, Type = Integer, Flags = &h0
		Stopping
		  Stopped
		  Pausing
		  Paused
		  Resuming
		  Starting
		Playing
	#tag EndEnum


	#tag ViewBehavior
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
