#tag Class
Protected Class radio_mgr
	#tag Method, Flags = &h0
		Sub Constructor()
		  
		  ' Initialize memory blocks used to control the radio.
		  '
		  pmbRadioIn  = New MemoryBlock(5)
		  pmbRadioOut = New MemoryBlock(5)
		  
		  ' Initialize the radio.
		  '
		  InitRadio
		  
		  ' Wait for radio to finish initializing.
		  '
		  WaitForRadioReady
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Convert_Frequency_To_PLL(intFrequency As UInt32, ByRef intPLL_Hi As UInt8, ByRef intPLL_Lo As UInt8)
		  
		  Dim intPLL_HiLo As UInt16
		  
		  Const IFREQUENCY = 225000 ' Intermidiate frequency
		  Const CFREQUENCY = 32768  ' Crystal frequency
		  
		  
		  ' Calculates a Phase-Lock Loop value from the specified MHz.
		  '
		  intPLL_HiLo = CType(((intFrequency + IFREQUENCY) * 4) / CFREQUENCY, UInt16)
		  
		  ' Get High byte of PLL_HiLo
		  '
		  intPLL_Hi = Bitwise.ShiftRight(intPLL_HiLo, 8, 16)
		  
		  ' Get Low byte of PLL_HiLo
		  '
		  intPLL_Lo = Bitwise.BitAnd(intPLL_HiLo, &hFF)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Convert_PLL_To_Frequency(intPLL_Hi As UInt8, intPLL_Lo As UInt8, ByRef intFrequency As UInt32)
		  
		  Dim intPLL_HiLo As UInt16
		  
		  Const IFREQUENCY = 225000 ' Intermidiate frequency
		  Const CFREQUENCY = 32768  ' Crystal frequency
		  
		  
		  ' Combine PLL_Hi and PLL_Lo bytes.
		  '
		  intPLL_HiLo = Bitwise.ShiftLeft(intPLL_Hi And &h3F, 8, 16) + intPLL_Lo
		  
		  ' Calculate MHz from Phase-Lock Loop values.
		  '
		  intFrequency  = (((intPLL_HiLo * CFREQUENCY) / 4) - IFREQUENCY)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Destructor()
		  
		  ' Mute the radio.
		  '
		  Set_RadioOut_Mute_Bit(True)
		  
		  RadioWrite
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function DumpRadioBits(WhichOnes As String) As String
		  
		  Dim strBitString As String
		  
		  strBitString = ""
		  
		  ' Dump all the bits!
		  '
		  Select Case WhichOnes.Uppercase
		    
		  Case "IN"
		    strBitString = strBitString + pmbRadioIn.UInt8Value(0).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioIn.UInt8Value(1).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioIn.UInt8Value(2).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioIn.UInt8Value(3).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioIn.UInt8Value(4).ToBinary(8)
		    
		  Case "OUT"
		    strBitString = strBitString + pmbRadioOut.UInt8Value(0).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioOut.UInt8Value(1).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioOut.UInt8Value(2).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioOut.UInt8Value(3).ToBinary(8)
		    strBitString = strBitString + "+"
		    strBitString = strBitString + pmbRadioOut.UInt8Value(4).ToBinary(8)
		    
		  End Select
		  
		  Return strBitString
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Frequency() As UInt32
		  
		  Dim intPLL_Hi As UInt8
		  Dim intPLL_Lo As UInt8
		  Dim intFrequency As UInt32
		  
		  
		  RadioRead
		  
		  Get_RadioIN_PLL_Bits(intPLL_Hi, intPLL_Lo)
		  Convert_PLL_To_Frequency(intPLL_Hi, intPLL_Lo, intFrequency)
		  
		  Return intFrequency
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Frequency(Assigns intFrequency As UInt32)
		  
		  Dim intPLL_Hi As UInt8
		  Dim intPLL_Lo As UInt8
		  
		  
		  Convert_Frequency_To_PLL(intFrequency, intPLL_Hi, intPLL_Lo)
		  Set_RadioOut_PLL_Bits(intPLL_Hi, intPLL_Lo)
		  
		  RadioWrite
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function Get_RadioIn_BandLimit_Bit() As Boolean
		  
		  ' Get bit 6 from mbRadioIn(0)
		  '
		  Return CType(pmbRadioIn.UInt8Value(0) And &h40, Boolean)
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Get_RadioIn_PLL_Bits(ByRef PLL_Hi As UInt8, ByRef PLL_Lo As UInt8)
		  
		  ' Get bits 5 thru 0 from mbRadioIn(0)
		  '
		  PLL_Hi = pmbRadioIn.Int8Value(0) And &h3F
		  
		  ' Get bits 7 thru 0 from mbRadioIn(1)
		  '
		  PLL_Lo = pmbRadioIn.Int8Value(1)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function Get_RadioIn_Ready_Bit() As Boolean
		  
		  ' Get bit 7 from mbRadioIn(0)
		  '
		  Return CType(pmbRadioIn.UInt8Value(0) And &h80, Boolean)
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function Get_RadioIn_SignalStrength_Bits() As Integer
		  
		  ' Get bits 7 thru 4 from mbRadioIn(3)
		  '
		  Return Bitwise.ShiftRight(pmbRadioIn.UInt8Value(3), 4, 8)
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Function Get_RadioIn_StereoIndicator_Bit() As Boolean
		  
		  ' Get bit 7 from mbRadioIn(2)
		  '
		  Return CType(pmbRadioIn.UInt8Value(2) And &h80, Boolean)
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub InitRadio()
		  
		  ' Initialize the radio.
		  '
		  pmbRadioOut.Int8Value(0) = Integer.FromBinary("10101001") ' 7=Mute, 6=Search Mode, 5->0=PLL
		  pmbRadioOut.Int8Value(1) = Integer.FromBinary("11010100") ' 7->0=PLL
		  pmbRadioOut.Int8Value(2) = Integer.FromBinary("11110000") ' 7=Search Up/Down, 6=Search Stop Level, 5=Search Stop Level, 4=HiLo Side Injection, 3=Forced Mono, 2=Mute Left, 1=Mute Right, 0=SW Port1 
		  pmbRadioOut.Int8Value(3) = Integer.FromBinary("00011110") ' 7=SW Port2, 6=Stand-By, 5=Band Limits, 4=XTal, 3=Soft-Mute, 2=HCC, 1=SNC, 0=Search Indicator
		  pmbRadioOut.Int8Value(4) = Integer.FromBinary("00000000") ' 7=PLL Ref, 6=Deemph, 5=Not used, 4=Not used, 3=Not used, 2=Not used, 1=Not used, 0=Not used
		  
		  RadioWrite
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Mute() As Boolean
		  
		  Return pbooMute
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Mute(Assigns booValue As Boolean)
		  
		  pbooMute = booValue
		  
		  Set_RadioOut_Mute_Bit(pbooMute)
		  
		  RadioWrite
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub RadioRead()
		  
		  Dim intRadioFD As Integer
		  Dim objBinaryStream As BinaryStream
		  
		  
		  ' Get file descriptor of radio.
		  '
		  intRadioFD = GPIO.I2CSetup(&h60)
		  
		  ' Open a binary stream to the radio.
		  '
		  objBinaryStream = New BinaryStream(intRadioFD, BinaryStream.HandleTypeFileNumber)
		  
		  ' Read 5 byte data packet from radio.
		  '
		  pmbRadioIn = objBinaryStream.Read(5)
		  
		  ' Did we read a full packet?
		  '
		  If pmbRadioIn.Size <> 5 Then
		    pmbRadioIn.Size = 5
		  End If
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub RadioWrite()
		  
		  Dim intRadioFD As Integer
		  Dim objBinaryStream As BinaryStream
		  
		  
		  ' Get file descriptor of radio.
		  '
		  intRadioFD = GPIO.I2CSetup(&h60)
		  
		  ' Open a binary stream to the radio.
		  '
		  objBinaryStream = New BinaryStream(intRadioFD, BinaryStream.HandleTypeFileNumber)
		  
		  ' Write data to radio.
		  '
		  objBinaryStream.Write(pmbRadioOut)
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub Search()
		  
		  Dim intPLL_Hi As UInt8
		  Dim intPLL_Lo As UInt8
		  Dim intFrequency As UInt32
		  
		  
		  ' Read data from radio.
		  '
		  RadioRead
		  
		  ' Get PLL bits for the currently tuned frequency.
		  '
		  Get_RadioIn_PLL_Bits(intPLL_Hi, intPLL_Lo)
		  
		  ' Convert PLL bits to frequency.
		  ' 
		  Convert_PLL_To_Frequency(intPLL_Hi, intPLL_Lo, intFrequency)
		  
		  ' Depending on search direction flag either increment
		  ' or decrement the MHz value by 0.1
		  '
		  Select Case pintSearchDirection
		  Case SEARCH_BACKWARD
		    intFrequency = intFrequency - 100000
		    If intFrequency <  87500000 Then intFrequency = 107900000
		  Case SEARCH_FORWARD
		    intFrequency = intFrequency + 100000
		    If intFrequency > 107900000 Then intFrequency =  87500000
		  End Select
		  
		  ' Generate new PLL_Hi and PLL_Lo bits from frequency.
		  '
		  Convert_Frequency_To_PLL(intFrequency, intPLL_Hi, intPLL_Lo)
		  
		  ' Now set the conditions needed for the radio to find the next usable station.
		  '
		  Set_RadioOut_PLL_Bits(intPLL_Hi, intPLL_Lo)
		  Set_RadioOut_SearchMode_Bit(True)
		  Set_RadioOut_Mute_Bit(True)
		  Select Case pintSearchDirection
		  Case SEARCH_BACKWARD
		    Set_RadioOut_SearchDirection_Bit(SEARCH_BACKWARD)
		  Case SEARCH_FORWARD
		    Set_RadioOut_SearchDirection_Bit(SEARCH_FORWARD)
		  End Select
		  
		  ' Update the radio.
		  '
		  RadioWrite
		  
		  ' Wait for the radio find a station.
		  '
		  WaitForRadioReady
		  
		  ' Get PLL bits for the currently tuned frequency.
		  '
		  Get_RadioIn_PLL_Bits(intPLL_Hi, intPLL_Lo)
		  
		  ' Set the PLL bits, and un-set the mute and search bits.
		  '
		  Set_RadioOut_PLL_Bits(intPLL_Hi, intPLL_Lo)
		  Set_RadioOut_SearchMode_Bit(False)
		  Set_RadioOut_Mute_Bit(False)
		  
		  ' Update the radio.
		  '
		  RadioWrite
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Sub SearchDirection(Assigns intValue As Integer)
		  
		  pintSearchDirection = intValue
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Set_RadioOut_Mute_Bit(Mute As Boolean)
		  
		  ' Set bit 7 of mbRadioOut(0)
		  '
		  Select Case Mute
		    
		  Case True
		    If (pmbRadioOut.Int8Value(0) AND &h80) = &h0 Then
		      pmbRadioOut.Int8Value(0) = pmbRadioOut.Int8Value(0) XOR &h80
		    End If
		    
		  Case False
		    If (pmbRadioOut.Int8Value(0) AND &h80) = &h80 Then
		      pmbRadioOut.Int8Value(0) = pmbRadioOut.Int8Value(0) XOR &h80
		    End If
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Set_RadioOut_PLL_Bits(PLL_Hi As UInt8, PLL_Lo As UInt8)
		  
		  Dim Bits7to6 As UInt8
		  Dim Bits5to0 As UInt8
		  
		  
		  ' PLL_Hi is bits 5 thru 0 of mbRadioOut(0), so don't disturb bits 7 and 6.
		  '
		  Bits7to6 = pmbRadioOut.Int8Value(0) And &hC0
		  Bits5to0 = PLL_Hi And &h3F
		  pmbRadioOut.Int8Value(0) = Bits7to6 + Bits5to0
		  
		  ' PLL_Lo is bits 7 thru 0 of mbRadioOut(1)
		  '
		  pmbRadioOut.Int8Value(1) = PLL_Lo
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Set_RadioOut_SearchDirection_Bit(Direction As Integer)
		  
		  ' Set bit 7 of mbRadioOut(2)
		  '
		  Select Case Direction
		    
		  Case SEARCH_FORWARD
		    If (pmbRadioOut.Int8Value(2) AND &h80) = &h0 Then
		      pmbRadioOut.Int8Value(2) = pmbRadioOut.Int8Value(2) XOR &h80
		    End If
		    
		  Case SEARCH_BACKWARD
		    If (pmbRadioOut.Int8Value(2) AND &h80) = &h80 Then
		      pmbRadioOut.Int8Value(2) = pmbRadioOut.Int8Value(2) XOR &h80
		    End If
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Set_RadioOut_SearchMode_Bit(Mode As Boolean)
		  
		  ' Set bit 6 of mbRadioOut(0)
		  '
		  Select Case Mode
		    
		  Case True
		    If (pmbRadioOut.Int8Value(0) AND &h40) = &h0 Then
		      pmbRadioOut.Int8Value(0) = pmbRadioOut.Int8Value(0) XOR &h40
		    End If
		    
		  Case False
		    If (pmbRadioOut.Int8Value(0) AND &h40) = &h40 Then
		      pmbRadioOut.Int8Value(0) = pmbRadioOut.Int8Value(0) XOR &h40
		    End If
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub Set_RadioOut_StandBy_Bit(StandBy As Boolean)
		  
		  ' Set bit 6 of mbRadioOut(3)
		  '
		  Select Case StandBy
		    
		  Case True
		    If (pmbRadioOut.Int8Value(3) AND &h40) = &h0 Then
		      pmbRadioOut.Int8Value(3) = pmbRadioOut.Int8Value(3) XOR &h40
		    End If
		    
		  Case False
		    If (pmbRadioOut.Int8Value(3) AND &h40) = &h40 Then
		      pmbRadioOut.Int8Value(3) = pmbRadioOut.Int8Value(3) XOR &h40
		    End If
		    
		  End Select
		  
		End Sub
	#tag EndMethod

	#tag Method, Flags = &h0
		Function SignalStrength() As Integer
		  
		  Return Get_RadioIn_SignalStrength_Bits
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h0
		Function Stereo() As Boolean
		  
		  Return Get_RadioIn_StereoIndicator_Bit
		  
		End Function
	#tag EndMethod

	#tag Method, Flags = &h21
		Private Sub WaitForRadioReady()
		  
		  Dim intLoopCount As Integer
		  
		  Do
		    
		    ' Pause for 1/10th of a second.
		    '
		    GPIO.Delay(100)
		    
		    ' Exit if 10 seconds has elapsed.
		    '
		    intLoopCount = intLoopCount + 1
		    If intLoopCount > 100 Then
		      Exit Do
		    End If
		    
		    ' Exit if the radio is ready.
		    '
		    RadioRead
		    If Get_RadioIn_Ready_Bit() = True Then
		      Exit Do
		    End If
		    
		  Loop
		  
		End Sub
	#tag EndMethod


	#tag Note, Name = TEA5767
		
		' Philips Semiconductors
		' TEA5767 Low voltage FM stereo radio
		'
		' COMMUNICATION: WRITE MODE
		' +-------------+----------------+-------------------+-------------+---------------------+-------------+-----------+------------+------------------+
		' |             | Bit 7          | Bit 6             | Bit 5       | Bit 4               | Bit 3       | Bit 2     | Bit 1      | Bit 0            |
		' +-------------+----------------+-------------------+-------------+---------------------+-------------+-----------+------------+------------------+
		' | Data byte 1 | Mute           | Search mode       | PLL13       | PLL12               | PLL11       | PLL10     | PLL9       | PLL8             |
		' +-------------+----------------+-------------------+-------------+---------------------+-------------+-----------+------------+------------------+
		' | Data byte 2 | PLL7           | PLL6              | PLL5        | PLL4                | PLL3        | PLL2      | PLL1       | PLL0             |
		' +-------------+----------------+-------------------+-------------+---------------------+-------------+-----------+------------+------------------+
		' | Data byte 3 | Search up/down | Search stop level |             | HiLo side injection | Mono/stereo | Mute left | Mute right | SW port1         |
		' +-------------+----------------+-------------------+-------------+---------------------+-------------+-----------+------------+------------------+
		' | Data byte 4 | SW port 2      | Stand-by          | Band limits | xtal                | Soft-mute   | HCC       | SNC        | Search indicator |
		' +-------------+----------------+-------------------+-------------+---------------------+-------------+-----------+------------+------------------+
		' | Data byte 5 | PLL ref        | Deemph            | Not used    | Not used            | Not used    | Not used  | Not used   | Not used         |
		' +-------------+----------------+-------------------+-------------+---------------------+-------------+-----------+------------+------------------+
		'
		' +-------------------------+-------------------------------------------------+
		' | Data                    | Description                                     |
		' +-------------------------+-------------------------------------------------+
		' | Mute                    | 1: L-, R-audio muted                            |
		' |                         | 0: Audio not muted                              |
		' +-------------------------+-------------------------------------------------+
		' | Search Mode             | 1: Search mode                                  |
		' |                         | 0: Not in search mode                           |
		' +-------------------------+-------------------------------------------------+
		' | PLL13...PLL0            | Setting of the synthesizer programmable counter |
		' +-------------------------+-------------------------------------------------+
		' | Search Up/Down          | 1: Search up                                    |
		' |                         | 0: Search down                                  |
		' +-------------------------+-------------------------------------------------+
		' | Search stop level       | See table 4 (page 29)                           |
		' +-------------------------+-------------------------------------------------+
		' | High/Low side injection | 1: High side LO injection                       |
		' |                         | 0: Low side LO injection                        |
		' +-------------------------+-------------------------------------------------+
		' | Mono/Stereo             | 1: Forced mono                                  |
		' |                         | 0: Stereo on                                    |
		' +-------------------------+-------------------------------------------------+
		' | Mute Left               | 1: Left audio muted and forced mono             |
		' |                         | 0: Not muted                                    |
		' +-------------------------+-------------------------------------------------+
		' | Mute Right              | 1: Right audio muted and forced mono            |
		' |                         | 0: Not muted                                    |
		' +-------------------------+-------------------------------------------------+
		' | SW port 1 & SW port 2   | 1: Port high                                    |
		' |                         | 0: Port low                                     |
		' +-------------------------+-------------------------------------------------+
		' | Stand-by                | 1: Tuner in stand-by                            |
		' |                         | 0: Not in stand-by                              |
		' +-------------------------+-------------------------------------------------+
		' | Band limits (BL)        | 1: Japan FM band                                |
		' |                         | 0: US/Europe FM band                            |
		' +-------------------------+-------------------------------------------------+
		' | Xtal                    | 1: Fxtal is 32.768kHz                           |
		' |                         | 0: Fxtal is 13MHz                               |
		' +-------------------------+-------------------------------------------------+
		' | Soft_mute               | 1: On                                           |
		' |                         | 0: Off                                          |
		' +-------------------------+-------------------------------------------------+
		' | HCC                     | 1: On                                           |
		' |                         | 0: Off                                          |
		' +-------------------------+-------------------------------------------------+
		' | SNC                     | 1: On                                           |
		' |                         | 0: Off                                          |
		' +-------------------------+-------------------------------------------------+
		' | Search indicator        | 1: Pin 14 is output for the ready flag          |
		' |                         | 0: Pin 14 is SW port 1                          |
		' +-------------------------+-------------------------------------------------+
		' | PLL ref                 | 1: 6.5MHz reference for PLL enabled             |
		' |                         | 0: 6.5MHz reference not enabled                 |
		' +-------------------------+-------------------------------------------------+
		' | Deemph                  | 1: Deemphasis time constant is 75us             |
		' |                         | 0: Deemphasis time constant is 50us             |
		' +-------------------------+-------------------------------------------------+
		
		
		' COMMUNICATION: READ MODE
		' +-------------+-------------------+-----------------+-------------+---------+----------+----------+----------+----------+
		' |             | Bit 7             | Bit 6           | Bit 5       | Bit 4   | Bit 3    | Bit 2    | Bit 1    | Bit 0    |
		' +-------------+-------------------+-----------------+-------------+---------+----------+----------+----------+----------+
		' | Data byte 1 | Ready flag        | Band limit flag | PLL13       | PLL12   | PLL11    | PLL10    | PLL9     | PLL8     |
		' +-------------+-------------------+-----------------+-------------+---------+----------+----------+----------+----------+
		' | Data byte 2 | PLL7              | PLL6            | PLL5        | PLL4    | PLL3     | PLL2     | PLL1     | PLL0     |
		' +-------------+-------------------+-----------------+-------------+---------+----------+----------+----------+----------+
		' | Data byte 3 | Stereo indication | IF6             | IF5         | IF4     | IF3      | IF2      | IF1      | IF0      |
		' +-------------+-------------------+-----------------+-------------+---------+----------+----------+----------+----------+
		' | Data byte 4 | LEV3              | LEV2            | LEV1        | LEV0    | Chip ID2 | Chip ID1 | Chip ID0 | Not used |
		' +-------------+-------------------+-----------------+-------------+---------+----------+----------+----------+----------+
		' | Data byte 5 | Reserve           | Reserve         | Reserve     | Reserve | Reserve  | Reserve  | Reserve  | Reserve  |
		' +-------------+-------------------+-----------------+-------------+---------+----------+----------+----------+----------+
		'
		' +-------------------+-------------------------------------------------+
		' | Data              | Description                                     |
		' +-------------------+-------------------------------------------------+
		' | Ready flag        | 1: Tuning completed or BL reached               |
		' |                   | 0: Busy                                         |
		' +-------------------+-------------------------------------------------+
		' | Band limit flag   | 1: Band limit reached                           |
		' |                   | 0: Band limit not reached                       |
		' +-------------------+-------------------------------------------------+
		' | PLL13...PLL0      | Setting of the synthesizer programmable counter |
		' +-------------------+-------------------------------------------------+
		' | Stereo indication | 1: Stereo reception                             |
		' |                   | 0: Mono reception                               |
		' +-------------------+-------------------------------------------------+
		' | IF6...IF0         | IF counter result                               |
		' +-------------------+-------------------------------------------------+
		' | LEV3...LEV0       | Level ADC output                                |
		' +-------------------+-------------------------------------------------+
		
		
	#tag EndNote


	#tag Property, Flags = &h21
		Private pbooMute As Boolean = False
	#tag EndProperty

	#tag Property, Flags = &h21
		Private pintSearchDirection As Integer = SEARCH_FORWARD
	#tag EndProperty

	#tag Property, Flags = &h21
		Private pmbRadioIn As MemoryBlock
	#tag EndProperty

	#tag Property, Flags = &h21
		Private pmbRadioOut As MemoryBlock
	#tag EndProperty


	#tag Constant, Name = SEARCH_BACKWARD, Type = Double, Dynamic = False, Default = \"0", Scope = Public
	#tag EndConstant

	#tag Constant, Name = SEARCH_FORWARD, Type = Double, Dynamic = False, Default = \"1", Scope = Public
	#tag EndConstant


	#tag ViewBehavior
		#tag ViewProperty
			Name="Index"
			Visible=true
			Group="ID"
			InitialValue="-2147483648"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Left"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Name"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Super"
			Visible=true
			Group="ID"
			Type="String"
		#tag EndViewProperty
		#tag ViewProperty
			Name="Top"
			Visible=true
			Group="Position"
			InitialValue="0"
			Type="Integer"
		#tag EndViewProperty
	#tag EndViewBehavior
End Class
#tag EndClass
