#tag Class
Protected Class comvette
Inherits Application
	#tag Event
		Sub Close()
		  
		  ' Destroy main window.
		  '
		  gobjMainWindow = Nil
		  
		  ' Destroy the global instance of the Audio Switch Manager class.
		  '
		  gobjAudioSwitchMgr = Nil
		  
		  ' Destroy the global instance of the Antenna Manager class.
		  '
		  gobjAntennaMgr = Nil
		  
		  ' Destroy the global instance of the Amplifier Manager class.
		  '
		  gobjAmplifierMgr = Nil
		  
		End Sub
	#tag EndEvent

	#tag Event
		Sub Open()
		  
		  ' Get path to our executable
		  '
		  gstrAppPath = GetFolderItem("").NativePath
		  
		  ' Get our executable file name
		  '
		  gstrAppName = App.ExecutableFile.Name
		  
		  ' Get folder item for INI file
		  '
		  gobjINI_File = New FolderItem(gstrAppPath + gstrAppName + ".ini")
		  
		  ' Load configuration
		  '
		  ConfigLoad
		  
		  ' Initialize the GPIO system.
		  '
		  GPIO.SetupGPIO
		  
		  ' Set the piezo beeper GPIO pin to act as an output.
		  '
		  GPIO.PinMode(gintConfig_PinBeeper, GPIO.OUTPUT)
		  
		  ' Create a global instance of the Audio Switch Manager class.
		  '
		  gobjAudioSwitchMgr = New audio_switch_mgr
		  
		  ' Create a global instance of the Antenna Manager class.
		  '
		  gobjAntennaMgr = New antenna_mgr
		  
		  ' Create a global instance of the Amplifier Manager class.
		  '
		  gobjAmplifierMgr = New amplifier_mgr
		  
		  ' Hide the mouse pointer.
		  '
		  App.MouseCursor = System.Cursors.InvisibleCursor
		  
		  ' Create main window.
		  '
		  gobjMainWindow = New main_window
		  
		  ' Display main window.
		  '
		  gobjMainWindow.Show
		  
		End Sub
	#tag EndEvent

	#tag Event
		Function UnhandledException(error As RuntimeException) As Boolean
		  
		  Dim objFolderItem As New FolderItem
		  Dim objTextOutputStream As TextOutputStream
		  Dim objDate As New Date
		  Dim strDumpTxt As String
		  Dim strErrorType As String
		  Dim strStageCode() As String = Array("Development","Alpha","Beta","Final")
		  
		  
		  objFolderItem = GetFolderItem(gstrAppPath + "comvette.log", FolderItem.PathTypeNative)
		  
		  strErrorType = Introspection.GetType(Error).Name
		  
		  strDumpTxt = ""
		  strDumpTxt = strDumpTxt + Repeat("=", 120)                                         + EndOfLine
		  strDumpTxt = strDumpTxt + "       Error Date: " + objDate.SQLDateTime.ToText       + EndOfLine
		  strDumpTxt = strDumpTxt + "       Error Type: " + strErrorType                     + EndOfLine
		  strDumpTxt = strDumpTxt + "    Error Message: " + Error.Message                    + EndOfLine
		  strDumpTxt = strDumpTxt + "      App Version: " + app.MajorVersion.ToText          + "."
		  strDumpTxt = strDumpTxt +                         app.MinorVersion.ToText          + "."
		  strDumpTxt = strDumpTxt +                         app.BugVersion.ToText            + "."
		  strDumpTxt = strDumpTxt +                         app.NonReleaseVersion.ToText     + EndOfLine
		  strDumpTxt = strDumpTxt + "   App Stage Code: " + strStageCode(app.StageCode)      + EndOfLine
		  strDumpTxt = strDumpTxt + "App Short Version: " + app.ShortVersion                 + EndOfLine
		  strDumpTxt = strDumpTxt + " App Long Version: " + app.LongVersion                  + EndOfLine
		  strDumpTxt = strDumpTxt + "       Build Date: " + app.BuildDate.SQLDateTime.ToText + EndOfLine
		  strDumpTxt = strDumpTxt                                                            + EndOfLine
		  strDumpTxt = strDumpTxt + "       Call Stack: "                                    + EndOfLine
		  
		  For Each Element As String In Error.stack
		    strDumpTxt = strDumpTxt + Repeat(" ", 19) + Element + EndOfLine
		  Next
		  
		  objTextOutputStream = TextOutputStream.Append(objFolderItem)
		  objTextOutputStream.Write ConvertEncoding(strDumpTxt, Encodings.ASCII)
		  objTextOutputStream.Close
		  
		  MsgBox("An unhandled error occured." + EndOfLine + EndOfLine + "Error type: " + strErrorType + EndOfLine + "Error info written to: " + objFolderItem.DisplayName + EndOfLine + EndOfLine + "Application will be closed.")
		  
		  Quit
		  
		End Function
	#tag EndEvent


	#tag Note, Name = Notice
		This file is part of Comvette.
		
		Comvette is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.
		
		Comvette is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public License
		along with Comvette.  If not, see <https://www.gnu.org/licenses/>.
		
	#tag EndNote

	#tag Note, Name = ReadMe
		The Comvette program is an integral part of a larger project outlined on my
		website:
		
		  https://www.lifeofpi.blog
		
		I wanted to create an infotainment system for my classic 4th generation
		Corvette (1984-1989) which would replace the aging in-dash BOSE AM/FM/Cassette
		stereo with something more modern. Comvette is the result of my efforts.
		
		I am open sourcing the software under the GNU GPL in the hopes that others
		will find it useful and possibly contribute to its further development.
		
		Comvette is an evolving project so check in once in a while to see the latest
		developments.
		
		If you have comments, questions or suggestions you can contact me at:
		
		  wes@westhaver.com
		
		Comvette is Copyright 2017-2019 Lawrence Westhaver.
		
		
	#tag EndNote

	#tag Note, Name = To Do List
		To Do List:
		
		Close full-screen video backdrop window when end of playlist is hit.
		
		Check if file exists before trying to pass it to the OMX player.
		
		Check for no selection in Music and Movie playlist selector dialog.
		
		Finish the GPS Recorder code to log GPS data and allow comments to be added to logs.
		
		Create an OBD1 reader (Arduino) and a Diagnostics display page (Xojo).
		
		
	#tag EndNote


	#tag ViewBehavior
	#tag EndViewBehavior
End Class
#tag EndClass
