#tag Class
Protected Class shutdown
Inherits ConsoleApplication
	#tag Event
		Function Run(args() as String) As Integer
		  
		  ' Get path to our executable.
		  '
		  gstrAppPath = GetFolderItem("").NativePath
		  
		  ' Get our executable file name.
		  '
		  gstrAppName = App.ExecutableFile.Name
		  
		  ' Get folder item for INI file.
		  '
		  gobjINI_File = New FolderItem(gstrAppPath + gstrAppName + ".ini")
		  
		  ' Load configuration.
		  '
		  ConfigLoad
		  
		  
		  ' Initialize the GPIO system.
		  '
		  GPIO.SetupGPIO
		  
		  ' Configure the shutdown request GPIO pin to act as an input.
		  '
		  GPIO.PinMode(gintConfig_PinShutdownRequest, GPIO.Input)
		  
		  ' Configure the shutdown request GPIO pin to have a pull-down resistor.
		  '
		  GPIO.PullUpDnControl(gintConfig_PinShutdownRequest, GPIO.PUD_DOWN)
		  
		  
		  ' Display operating parameters.
		  '
		  stdout.WriteLine("Raspberry Pi Shutdown Request Monitor.")
		  stdout.WriteLine("")
		  
		  stdout.WriteLine(" Daemonize = "              + Str(gbooConfig_Daemonize))
		  stdout.WriteLine(" Shutdown Request Pin = "   + Str(gintConfig_PinShutdownRequest))
		  stdout.WriteLine(" Polling Frequency (ms) = " + Str(gintConfig_PollingFrequency))
		  stdout.WriteLine(" Shutdown Command = "       + Str(gstrConfig_ShutdownCommand))
		  stdout.WriteLine("")
		  
		  stdout.Write(" Polling: ( )")
		  
		  
		  ' Create timer to check every n seconds for shutdown request.
		  '
		  Dim objTimer As Timer = New Timer
		  objTimer.Mode = Timer.ModeMultiple
		  objTimer.Period = gintConfig_PollingFrequency
		  AddHandler objTimer.Action, AddressOf TimerAction
		  
		  
		  ' Run as a daemon?
		  '
		  If gbooConfig_Daemonize = True Then
		    Dim booRetVal As Boolean = App.Daemonize
		    If booRetVal = False Then
		      stdout.WriteLine("ERROR: Could not daemonize the application.")
		      Return -1
		    End If
		  End If
		  
		  
		  ' Start processing loop.
		  '
		  Do
		    App.DoEvents
		  Loop
		  
		End Function
	#tag EndEvent

	#tag Event
		Function UnhandledException(error As RuntimeException) As Boolean
		  
		  Dim objFolderItem As New FolderItem
		  Dim objTextOutputStream As TextOutputStream
		  Dim objDate As New Date
		  Dim strDumpTxt As String
		  Dim strErrorType As String
		  Dim strStageCode() As String = Array("Development","Alpha","Beta","Final")
		  
		  
		  objFolderItem = GetFolderItem(gstrAppPath + "shutdown.log", FolderItem.PathTypeNative)
		  
		  strErrorType = Introspection.GetType(Error).Name
		  
		  strDumpTxt = ""
		  strDumpTxt = strDumpTxt + Repeat("=", 120)                                         + EndOfLine
		  strDumpTxt = strDumpTxt + "       Error Date: " + objDate.SQLDateTime.ToText       + EndOfLine
		  strDumpTxt = strDumpTxt + "       Error Type: " + strErrorType                     + EndOfLine
		  strDumpTxt = strDumpTxt + "    Error Message: " + Error.Message                    + EndOfLine
		  strDumpTxt = strDumpTxt + "      App Version: " + app.MajorVersion.ToText          + "."
		  strDumpTxt = strDumpTxt +                         app.MinorVersion.ToText          + "."
		  strDumpTxt = strDumpTxt +                         app.BugVersion.ToText            + "."
		  strDumpTxt = strDumpTxt +                         app.NonReleaseVersion.ToText     + EndOfLine
		  strDumpTxt = strDumpTxt + "   App Stage Code: " + strStageCode(app.StageCode)      + EndOfLine
		  strDumpTxt = strDumpTxt + "App Short Version: " + app.ShortVersion                 + EndOfLine
		  strDumpTxt = strDumpTxt + " App Long Version: " + app.LongVersion                  + EndOfLine
		  strDumpTxt = strDumpTxt + "       Build Date: " + app.BuildDate.SQLDateTime.ToText + EndOfLine
		  strDumpTxt = strDumpTxt                                                            + EndOfLine
		  strDumpTxt = strDumpTxt + "       Call Stack: "                                    + EndOfLine
		  
		  For Each Element As String In Error.stack
		    strDumpTxt = strDumpTxt + Repeat(" ", 19) + Element + EndOfLine
		  Next
		  
		  objTextOutputStream = TextOutputStream.Append(objFolderItem)
		  objTextOutputStream.Write ConvertEncoding(strDumpTxt, Encodings.ASCII)
		  objTextOutputStream.Close
		  
		  MsgBox("An unhandled error occured." + EndOfLine + EndOfLine + "Error type: " + strErrorType + EndOfLine + "Error info written to: " + objFolderItem.DisplayName + EndOfLine + EndOfLine + "Application will be closed.")
		  
		  Quit
		  
		End Function
	#tag EndEvent


	#tag Note, Name = Notice
		This file is part of Comvette.
		
		Comvette is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.
		
		Comvette is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.
		
		You should have received a copy of the GNU General Public License
		along with Comvette.  If not, see <https://www.gnu.org/licenses/>.
		
		
	#tag EndNote

	#tag Note, Name = ReadMe
		This program (Shutdown) is a daemon to monitor a user defined GPIO pin for
		a shutdown request. When the shutdown request is received, it shells out 
		and issues a user defined command to the OS to shutdown.
		
		Shutdown is Copyright 2017-2019 Lawrence Westhaver.
		
	#tag EndNote


	#tag ViewBehavior
	#tag EndViewBehavior
End Class
#tag EndClass
